package com.dragon.tj.portal.component.message;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dragon.tj.portal.common.convert.mesage.MessageReqConvert;
import com.dragon.tj.portal.common.convert.notice.NoticeReqConvert;
import com.dragon.tj.portal.common.dto.message.MessageInfoSend;
import com.dragon.tj.portal.common.dto.notice.NoticeInfoSend;
import com.dragon.tj.portal.common.enums.message.MsgRecordTypeEnum;
import com.dragon.tj.portal.entity.MessageInfo;
import com.dragon.tj.portal.entity.NoticeInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Map;
import java.util.Set;

/**
 * @author huey China.
 * @Description : 消费端
 * @Date Created in 2023/6/15 15:42
 */
@Component
@Slf4j
public class MessageConsumer {

    @Autowired
    private Map<String, OpenApiSseEmitter> sseEmitters;

    @Autowired
    private MessageReqConvert messageReqConvert;

    @Autowired
    private NoticeReqConvert noticeReqConvert;

    /**
     * get client published message from kafka，process it according to the serverId
     *
     * @param msg
     * @param ack
     */
    @KafkaListener(topics = KafkaInitialConfiguration.sseTopic)
    public void sseConnectionProcess(String msg, Acknowledgment ack) {
        log.info("get kafka msg from topic:{}, msg:{}", KafkaInitialConfiguration.sseTopic, msg);
        if (StrUtil.isEmpty(msg)) {
            log.error("kafka msg is empty, no process");
            return;
        }
        MessageInfoSend reqDTO;
        try {
            reqDTO = JSONObject.parseObject(msg, MessageInfoSend.class);
        } catch (Exception e) {
            log.error("parsing string to obj failed, msg={}, e={}", msg, e);
            return;
        }
        Set<String> clientIds = reqDTO.getClientIds();
        MessageInfo messageInfo = reqDTO.getMessageInfo();

        for (String clientId : clientIds) {
            OpenApiSseEmitter emitter = sseEmitters.get(clientId);
            if (emitter == null) {
                log.error("can't find the sseEmitter obj from sseEmitters, no process {}", clientId);
                continue;
            }
            try {
                messageInfo.setClientId(clientId);
                messageInfo.setRecordType(MsgRecordTypeEnum.MESSAGE.value());

                String sendMsg = JSON.toJSONString(messageInfo);
                // 发送消息给客户端
                log.info("send sse msg={} to clientId={}", sendMsg, clientId);
                emitter.send(sendMsg, MediaType.APPLICATION_JSON);
            } catch (IOException e) {
                emitter.completeWithError(e);
            }
        }
        ack.acknowledge();
    }

    @KafkaListener(topicPattern = "sseTopic.*")
    public void sseConnectionProcess2(String msg, Acknowledgment ack) {
        log.info("get kafka2 msg from topic:{}, msg:{}", KafkaInitialConfiguration.sseTopic, msg);
        if (StrUtil.isEmpty(msg)) {
            log.error("kafka msg is empty, no process");
            return;
        }
        NoticeInfoSend reqDTO;
        try {
            reqDTO = JSONObject.parseObject(msg, NoticeInfoSend.class);
        } catch (Exception e) {
            log.error("parsing string to obj failed, msg={}, e={}", msg, e);
            return;
        }
        Set<String> clientIds = reqDTO.getClientIds();
        NoticeInfo noticeInfo = reqDTO.getNoticeInfo();

        for (String clientId : clientIds) {
            OpenApiSseEmitter emitter = sseEmitters.get(clientId);
            if (emitter == null) {
                log.error("can't find the sseEmitter obj from sseEmitters, no process {}", clientId);
                continue;
            }
            try {
                noticeInfo.setClientId(clientId);
                noticeInfo.setRecordType(MsgRecordTypeEnum.NOTICE.value());

                String sendMsg = JSON.toJSONString(noticeInfo);
                // 发送消息给客户端
                log.info("send sse msg={} to clientId={}", sendMsg, clientId);
                emitter.send(sendMsg, MediaType.APPLICATION_JSON);
            } catch (IOException e) {
                emitter.completeWithError(e);
            }
        }
        ack.acknowledge();
    }
}
